package ch.unige.solidify.test.util;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.core.io.ClassPathResource;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import gov.loc.repository.bagit.domain.Bag;

import ch.unige.solidify.util.BagItTool;
import ch.unige.solidify.util.FileTool;

@ExtendWith(SpringExtension.class)
class BagItTest {

  private final static String BAGIT_FOLDER = "bagit";
  private final static String BAG_FOLDER = "bag";

  @AfterEach
  public void purgeData() throws IOException {
    FileTool.deleteFolder(Paths.get(new ClassPathResource(BAGIT_FOLDER).getURI()).resolve(BAG_FOLDER));
  }

  @Test
  void createTest() throws IOException {
    Path bagFolder = this.createBag();
    Bag bag = BagItTool.create(bagFolder, null);
    this.verifyBag(bag);
  }

  @Test
  void readTest() throws IOException {
    Path bagFolder = this.createBag();
    BagItTool.create(bagFolder, this.getMetedata());
    Bag bag = BagItTool.read(bagFolder.toString());
    this.verifyBag(bag);
  }

  @Test
  void checkTest() throws IOException {
    Path bagFolder = this.createBag();
    Bag bag = BagItTool.create(bagFolder, null);
    BagItTool.check(null, bag);
  }

  @Test
  void quickCheckTest() {
    try {
      Path bagFolder = this.createBag();
      Bag bag = BagItTool.create(bagFolder, null);
      BagItTool.quickCheck(null, bag);
    } catch(RuntimeException|IOException exception) {
      fail("Exception " + exception.getClass() + " should not happen");
    }
  }

  private void verifyBag(Bag bag) {
    assertEquals("1.0", bag.getVersion().toString());
    assertEquals(2, BagItTool.getBagFileNumber(bag));
  }

  private Map<String, String> getMetedata() {
    HashMap<String, String> metadata = new HashMap<>();
    return metadata;
  }

  private Path createBag() throws IOException {
    Path source = Paths.get(new ClassPathResource(BAGIT_FOLDER).getURI());
    Path destination = source.resolve(BAG_FOLDER);
    for (Path f : FileTool.findFiles(source)) {
      FileTool.copyFile(f, destination.resolve(f.getFileName()));
    }
    return destination;
  }

}
