package ch.unige.solidify.service;

import java.security.Principal;
import java.text.ParseException;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.context.request.RequestContextHolder;

import com.nimbusds.jwt.SignedJWT;

@Profile("client-backend")
@Component
public class SolidifyHttpRequestInfoProvider implements HttpRequestInfoProvider {
  private static final Logger log = LoggerFactory.getLogger(SolidifyHttpRequestInfoProvider.class);

  private HttpServletRequest incomingRequest;

  private SolidifyHttpRequestInfoProvider(HttpServletRequest incomingRequest) {
    this.incomingRequest = incomingRequest;
  }

  @Override
  public MultiValueMap<String, String> getAuthorizationHeader() {
    final MultiValueMap<String, String> authorizationHeader = new LinkedMultiValueMap<>();
    authorizationHeader.add(HttpHeaders.AUTHORIZATION, "Bearer " + this.getIncomingToken());
    return authorizationHeader;
  }

  @Override
  public String getIncomingToken() {
    String token = null;
    if (RequestContextHolder.getRequestAttributes() != null) {
      log.debug("Request with attributes");
      if (this.incomingRequest.getHeader(HttpHeaders.AUTHORIZATION) != null) {
        token = this.incomingRequest.getHeader(HttpHeaders.AUTHORIZATION).substring("Bearer ".length());
        if (log.isDebugEnabled()) {
          try {
            log.debug("Authorization header present {}", SignedJWT.parse(token).getJWTClaimsSet());
          } catch (ParseException e) {
            log.debug("Invalid token, cannot parse token");
          }
        }
      } else {
        log.debug("Authorization header not present");
      }
    } else {
      log.debug("Request attributes is null");
    }
    return token;
  }

  @Override
  public Principal getPrincipal() {
    if (RequestContextHolder.getRequestAttributes() != null) {
      return this.incomingRequest.getUserPrincipal();
    }
    return null;
  }
}
