package ch.unige.solidify.config;

import java.net.URL;
import java.net.URLClassLoader;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Service;

import ch.unige.solidify.service.GitInfoService;

@Service
@ConfigurationProperties(prefix = "solidify")
public class SolidifyProperties {
  private static final Logger log = LoggerFactory.getLogger(SolidifyProperties.class);

  // ****************
  // ** Properties **
  // ****************
  @Autowired
  private GitInfoService gitInfoService;

  @Value("${server.servlet.application-display-name:Default Server Display Name}")
  private String serverDisplayName;

  private Email email = new Email();

  // *********************
  // ** Getters/Setters **
  // *********************
  public static Logger getLog() {
    return log;
  }

  public String getApplicationName() {
    return this.gitInfoService.getAppName();
  }

  public String getApplicationVersion() {
    return this.gitInfoService.getBuildVersion();
  }

  public GitInfoService getGitInfoService() {
    return this.gitInfoService;
  }

  public String getServerDisplayName() {
    return this.serverDisplayName;
  }

  public void logClassPath() {
    final ClassLoader cl = ClassLoader.getSystemClassLoader();
    final URL[] urls = ((URLClassLoader) cl).getURLs();
    log.info("List CLASSPATH");
    for (final URL url : urls) {
      log.info(url.getFile());
    }
  }

  public Email getEmail() {
    return this.email;
  }

  public void setEmail(Email email) {
    this.email = email;
  }

  // *************************
  // ** Embedded Properties **
  // *************************

  // Instance
  public static class Email {
    private String senderAddress;
    private String[] ccList = {};
    private String[] bccList = {};
    private String externalTemplates;
    private String internalTemplates = "email-templates/";

    public String getSenderAddress() {
      return this.senderAddress;
    }

    public void setSenderAddress(String senderAddress) {
      this.senderAddress = senderAddress;
    }

    public String getExternalTemplates() {
      return this.externalTemplates;
    }

    public String[] getCcList() {
      return this.ccList;
    }

    public void setCcList(String[] ccList) {
      this.ccList = ccList;
    }

    public String[] getBccList() {
      return this.bccList;
    }

    public void setBccList(String[] bccList) {
      this.bccList = bccList;
    }

    public void setExternalTemplates(String externalTemplates) {
      this.externalTemplates = externalTemplates;
    }

    public String getInternalTemplates() {
      return this.internalTemplates;
    }

    public void setInternalTemplates(String internalTemplates) {
      this.internalTemplates = internalTemplates;
    }
  }
}