package ch.unige.solidify.service;

import static ch.unige.solidify.util.StringTool.isNullOrEmpty;

import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.model.RoleApplication;
import ch.unige.solidify.rest.ResourceBase;

public abstract class AbstractResourceService {

  public void setAuthenticatedUserProperties(ResourceBase resourceBase) {
    String userId = this.getAuthenticatedUserId();
    if (userId == null) {
      userId = SolidifyConstants.NO_CREDENTIAL;
    }
    if (this.isHumanUser() || isNullOrEmpty(resourceBase.getUpdatedBy())) {
      resourceBase.setUpdatedBy(userId);
    }
    if (isNullOrEmpty(resourceBase.getCreatedBy())) {
      resourceBase.setCreatedBy(userId);
    }
  }

  public String getAuthenticatedUserId() {
    final Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

    if (authentication != null && !(authentication instanceof AnonymousAuthenticationToken)) {
      return authentication.getName();
    } else {
      return null;
    }
  }

  public boolean isHumanUser() {
    Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    if (authentication == null) {
      return false;
    }
    for (GrantedAuthority authority : authentication.getAuthorities()) {
      if (authority.getAuthority().equals(RoleApplication.TRUSTED_CLIENT_ID) ||
              authority.getAuthority().equals(SolidifyConstants.MESSAGE_PROCESSOR_ROLE)) {
        return false;
      }
    }
    return true;
  }
}
