package ch.unige.solidify.test.helper;

import static ch.unige.solidify.test.helper.AbstractResourceDoc.TEST_STRING_VALUE;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONObject;
import org.springframework.restdocs.payload.FieldDescriptor;

public class TestObjectBuilder {
  /**
   * Build a JSON object from the list of field descriptors Only field descriptors with TEST_VALUE
   * attribute are taken Embedded objects are constructed from path names with dotted notation
   *
   * @return a JSON object
   * @Param fieldDescriptorsList the list of the field descriptors
   */
  public static JSONObject buildJsonObject(List<FieldDescriptor> fieldDescriptorList) {
    final Map<String, Object> mainMap = new HashMap<>();
    for (final FieldDescriptor fieldDescriptor : fieldDescriptorList) {
      Map<String, Object> currentMap = mainMap;
      final String[] pathPartList = fieldDescriptor.getPath().split("\\.");
      for (int i = 0; i < pathPartList.length; i++) {
        if (currentMap.get(pathPartList[i]) == null) {
          if (i != pathPartList.length - 1) {
            final Map<String, Object> newMap = new HashMap<>();
            currentMap.put(pathPartList[i], newMap);
            currentMap = newMap;
          } else {
            currentMap.put(pathPartList[pathPartList.length - 1], fieldDescriptor.getAttributes().get(TEST_STRING_VALUE));
          }
        } else if (currentMap.get(pathPartList[i]) instanceof Map) {
          currentMap = (Map<String, Object>) currentMap.get(pathPartList[i]);
        }
      }

    }
    return new JSONObject(mainMap);
  }

}
