package ch.unige.solidify.config;

import org.springframework.context.annotation.Configuration;
import org.thymeleaf.spring5.SpringTemplateEngine;
import org.thymeleaf.templatemode.TemplateMode;
import org.thymeleaf.templateresolver.ClassLoaderTemplateResolver;
import org.thymeleaf.templateresolver.FileTemplateResolver;
import org.thymeleaf.templateresolver.ITemplateResolver;

import com.google.common.base.Charsets;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.util.StringTool;

@Configuration
public class EmailConfiguration {

  public EmailConfiguration(SolidifyProperties config, SpringTemplateEngine templateEngine) {
    // add template resolver for external templates
    if (!StringTool.isNullOrEmpty(config.getEmail().getExternalTemplates())) {
      templateEngine.addTemplateResolver(this.filesystemTemplateResolver(config.getEmail().getExternalTemplates()));
    }
    // Add template resolver for internal templates
    if (!StringTool.isNullOrEmpty(config.getEmail().getInternalTemplates())) {
      templateEngine.addTemplateResolver(this.emailTemplateResolver(config.getEmail().getInternalTemplates()));
    }
  }

  private ITemplateResolver filesystemTemplateResolver(String location) {
    FileTemplateResolver templateResolver = new FileTemplateResolver();
    templateResolver.setPrefix(location);
    templateResolver.setSuffix(SolidifyConstants.HTML_EXT);
    templateResolver.setTemplateMode(TemplateMode.HTML);
    templateResolver.setCharacterEncoding(Charsets.UTF_8.name());
    templateResolver.setOrder(0);
    templateResolver.setCheckExistence(true);
    templateResolver.setCacheable(false);
    return templateResolver;
  }

  private ITemplateResolver emailTemplateResolver(String location) {
    ClassLoaderTemplateResolver templateResolver = new ClassLoaderTemplateResolver();
    templateResolver.setPrefix(location);
    templateResolver.setSuffix(SolidifyConstants.HTML_EXT);
    templateResolver.setTemplateMode(TemplateMode.HTML);
    templateResolver.setCharacterEncoding(Charsets.UTF_8.name());
    templateResolver.setOrder(1);
    templateResolver.setCheckExistence(true);
    templateResolver.setCacheable(false);
    return templateResolver;
  }

}