package ch.unige.solidify.exception;

import java.util.List;

import ch.unige.solidify.validation.FieldValidationError;
import ch.unige.solidify.validation.ValidationError;

public class SolidifyValidationException extends SolidifyRuntimeException {
  private static final long serialVersionUID = 1L;

  private final transient ValidationError validationError;

  /***********************************/

  public SolidifyValidationException(String message, ValidationError error) {
    super(message);

    this.validationError = error;
  }

  public SolidifyValidationException(ValidationError error) {
    super("Validation failed");
    this.validationError = error;
  }

  /***********************************/

  public ValidationError getValidationError() {
    return this.validationError;
  }

  @Override
  public String toString() {

    final StringBuilder builder = new StringBuilder(
            this.getValidationError().getMainErrorMessage() + ": ");

    for (final FieldValidationError error : this.getValidationError().getErrors()) {
      final String fieldname = error.getFieldName();
      final List<String> errorMsgs = error.getErrorMessages();

      builder.append("[" + fieldname + ": " + String.join(", ", errorMsgs) + "]");
    }

    return builder.toString();
  }
}
