package ch.unige.solidify.rest;

import java.time.OffsetDateTime;
import java.util.Objects;
import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EntityListeners;
import javax.persistence.MappedSuperclass;

import org.springframework.data.jpa.domain.support.AuditingEntityListener;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.listener.CacheListener;
import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.model.ChangeInfo;
import ch.unige.solidify.model.CreatedChangeInfo;
import ch.unige.solidify.model.LastModifiedChangeInfo;

/**
 * Class that can inherited to represent a join table with additional properties
 *
 * @param <J>
 * @param <K>
 */
@MappedSuperclass
@EntityListeners({ CacheListener.class, AuditingEntityListener.class })
public abstract class JoinResource<J, K> extends ResourceBase {

  @AttributeOverride(name = "when", column = @Column(name = "creationWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "creationWho"))
  @Embedded
  protected CreatedChangeInfo creation = new CreatedChangeInfo();

  @AttributeOverride(name = "when", column = @Column(name = "lastUpdateWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "lastUpdateWho"))
  @Embedded
  protected LastModifiedChangeInfo lastUpdate = new LastModifiedChangeInfo();

  /*************************************************************************************/

  @Override
  public ChangeInfo getCreation() {
    return this.creation;
  }

  @Override
  @JsonIgnore
  public String getCreatedBy() {
    return this.getCreation().getWho();
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getCreationTime() {
    return this.getCreation().getWhen();
  }

  @Override
  public void setCreatedBy(String createdBy) {
    this.getCreation().setWho(createdBy);
  }

  /*************************************************************************************/

  @Override
  public ChangeInfo getLastUpdate() {
    return this.lastUpdate;
  }

  @Override
  @JsonIgnore
  public String getUpdatedBy() {
    return this.getLastUpdate().getWho();
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getUpdateTime() {
    return this.getLastUpdate().getWhen();
  }

  @Override
  public void setUpdatedBy(String updatedBy) {
    this.getLastUpdate().setWho(updatedBy);
  }

  @Override
  public void updateTime() {
    // Let the @LastModifiedDate annotation do the job
  }

  @JsonIgnore
  public CacheMessage getCacheMessage() {
    return null;
  }
  
  /*************************************************************************************/

  public abstract K getCompositeKey();

  public abstract void setCompositeKey(K compositeKey);

  /*************************************************************************************/

  @Override
  public boolean equals(final Object other) {
    if (this == other) {
      return true;
    }
    if (!(other instanceof JoinResource)) {
      return false;
    }
    if (!super.equals(other)) {
      return false;
    }
    final JoinResource<J, K> that = (JoinResource<J, K>) other;
    return Objects.equals(this.getCompositeKey(), that.getCompositeKey()) &&
            Objects.equals(this.getCreation(), that.getCreation()) &&
            Objects.equals(this.getLastUpdate(), that.getLastUpdate());
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.getCompositeKey(), this.getCreation(), this.getLastUpdate());
  }
}
