package ch.unige.solidify.util;

/**
 * Different operation types, used by SearchCriteria
 *
 * @see ch.unige.solidify.util.SearchCriteria
 */
public enum SearchOperation {
  BETWEEN, CONTAINS, ENDS_WITH, EQUALITY, GREATER_EQUAL, GREATER_THAN, LESS_EQUAL, LESS_THAN, NEGATION, ORACLE_CONTAINS, STARTS_WITH;

  public static final String CASE_INSENSITIVE_FLAG = "i";
  public static final String OR_PREDICATE_FLAG = "any";
  protected static final String[] OPERATION_SET = { ":", "!", ">", "<", "~", "¬", "§", "≥", "≤", "∞", "¢" };

  public static SearchOperation getOperation(final String input) {
    switch (input) {
      case ":":
        return EQUALITY;
      case "!":
        return NEGATION;
      case ">":
        return GREATER_THAN;
      case "<":
        return LESS_THAN;
      case "~":
        return CONTAINS;
      case "¬":
        return STARTS_WITH;
      case "§":
        return ENDS_WITH;
      case "≥":
        return GREATER_EQUAL;
      case "≤":
        return LESS_EQUAL;
      case "∞":
        return BETWEEN;
      case "¢":
        return ORACLE_CONTAINS;
      default:
        return null;
    }
  }

  public static String[] getOperationList() {
    return OPERATION_SET;
  }

}
