package ch.unige.solidify.test.specification;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.Optional;
import javax.annotation.Resource;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.unige.solidify.test.config.MockUserJpaConfig;
import ch.unige.solidify.test.specification.dao.EntityMock;
import ch.unige.solidify.test.specification.dao.EntityMockRepository;

@ExtendWith(SpringExtension.class)
@DataJpaTest
@ContextConfiguration(classes = { MockUserJpaConfig.class })
class ResourceTest {
  @Resource
  private EntityMockRepository entityMockRepository;

  @BeforeEach
  void setup() {
    final EntityMock entity = new EntityMock();
    entity.setEntityId("1");
    entity.setLabel("first");
    entity.setCreatedBy("unitTEST");
    this.entityMockRepository.save(entity);
  }

  @Test
  void Test_delete() {
    this.entityMockRepository.deleteById("1");
    final Optional<EntityMock> entity2 = this.entityMockRepository.findById("1");
    assertFalse(entity2.isPresent());
  }

  @Test
  void Test_insert_metaData() {

    final Optional<EntityMock> entity2 = this.entityMockRepository.findById("1");
    assertNotNull(entity2.get().getCreationTime());
    assertNotNull(entity2.get().getUpdateTime());
  }

  @Test
  void Test_update() {
    final Optional<EntityMock> entity = this.entityMockRepository.findById("1");
    final EntityMock entityMock = entity.get();
    entityMock.getUpdateTime();
    entityMock.setLabel("changed");
    this.entityMockRepository.save(entityMock);
    final Optional<EntityMock> entity2 = this.entityMockRepository.findById("1");
    assertEquals("changed", entity2.get().getLabel());
  }
}
