package ch.unige.solidify.util;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

import com.sun.xml.bind.marshaller.NamespacePrefixMapper;

public class XmlNamespacePrefixMapper extends NamespacePrefixMapper {

  private final Map<String, String> namespaces = new HashMap<>();

  public XmlNamespacePrefixMapper() {
    this.namespaces.put("http://www.w3.org/2001/XMLSchema-instance", "xsi");
    this.namespaces.put("http://www.w3.org/1999/xhtml", "xhtml");
    this.namespaces.put("http://www.w3.org/1999/xlink", "xlink");
  }

  @Override
  public String getPreferredPrefix(String namespaceUri, String suggestion, boolean requirePrefix) {
    return this.namespaces.getOrDefault(namespaceUri, suggestion);
  }

  protected void addSchema(String nameSpace, String nameSpacePrefix) {
    this.namespaces.put(nameSpace, nameSpacePrefix);
  }

  public Map<String, String> getNamespaces() {
    return this.namespaces;
  }

  public Map<String, String> getNamespacePrefixes() {
    return this.namespaces.entrySet().stream()
            .collect(Collectors.toMap(Map.Entry::getValue, Map.Entry::getKey));
  }

}
